package convertquery

import (
	"fmt"
	"query-service/internal/entity"
)

/* createConstraintStatements generates the appropriate amount of constraint lines calling createConstraingBoolExpression
Parameters: constraints is a list of constraintStructs that specify the constraints of a node or relation,
name is the id of the corresponding relation/node,
isRelation is a boolean specifying if this constraint comes from a node or relation

Return: a string containing a FILTER-statement with all the constraints
*/
func createConstraintStatements(constraints *[]entity.QueryConstraintStruct, name string, isRelation bool) *string {
	s := ""
	if len(*constraints) == 0 {
		return &s
	}

	newLineStatement := "\tFILTER"

	for _, v := range *constraints {
		s += fmt.Sprintf("%v %v \n", newLineStatement, *createConstraintBoolExpression(&v, name, isRelation))
		newLineStatement = "\tAND"
	}

	return &s
}

/* createConstraintBoolExpression generates a single boolean expression,
e.g. {name}.city == "New York".

Parameters: constraint is a single constraint of a node or relation,
name is the id of the corresponding relation/node,
isRelation is a boolean specifying if this constraint comes from a node or relation, that changes the structure of the expression

Return: a string containing an boolean expression of a single constraint
*/
func createConstraintBoolExpression(constraint *entity.QueryConstraintStruct, name string, isRelation bool) *string {
	var (
		match string
		value string
		line  string
	)

	// Constraint datatypes back end
	// text		  MatchTypes: EQ/NEQ/contains/excludes
	// number   MatchTypes: EQ/NEQ/GT/LT/GET/LET
	// bool     MatchTypes: EQ/NEQ

	switch constraint.DataType {
	case "text":
		value = fmt.Sprintf("\"%s\"", constraint.Value)
		switch constraint.MatchType {
		case "NEQ":
			match = "!="
		case "contains":
			match = "LIKE"
		case "excludes":
			match = "NOT LIKE"
			value = fmt.Sprintf("\"%s%%\"", constraint.Value)
		default: //EQ
			match = "=="
		}
	case "number":
		value = constraint.Value
		switch constraint.MatchType {
		case "NEQ":
			match = "!="
		case "GT":
			match = ">"
		case "LT":
			match = "<"
		case "GET":
			match = ">="
		case "LET":
			match = "<="
		default: //EQ
			match = "=="
		}
	default: /*bool*/
		value = constraint.Value
		switch constraint.MatchType {
		case "NEQ":
			match = "!="
		default: //EQ
			match = "=="
		}
	}

	if isRelation {
		line = fmt.Sprintf("%s.edges[*].%s ALL %s %s", name, constraint.Attribute, match, value)
	} else {
		line = fmt.Sprintf("%s.%s %s %s", name, constraint.Attribute, match, value)
	}
	return &line
}
